searchNodes=[{"ref":"Genex.html","title":"Genex","type":"behaviour","doc":"Genetic Algorithms in Elixir! Genex is a simple library for creating Genetic Algorithms in Elixir. A Genetic Algorithm is a search-based optimization technique based on the principles of Genetics and Natural Selection. The basic life-cycle of a Genetic Algorithm is as follows: 1) Initialize the Population 2) Loop until goal is reached a) Select Parents b) Perform Crossover c) Mutate some of population d) Select Survivors Genex uses a structure similar to the one above, using callbacks to give you full control over your genetic algorithm. Implementation Genex requires an implementation module: defmodule OneMax do use Genex def encoding do for _ &lt;- 1..15, do: Enum.random(0..1) end def fitness_function(chromosome), do: Enum.sum(chromosome.genes) def terminate?(population), do: population.max_fitness == 15 end Genex requires 3 function definitions: encoding/0, fitness_function/1, and terminate?/1. Let&#39;s take a closer look at each of these: Encoding def encoding do for _ &lt;- 1..15, do: Enum.random(0..1) end encoding/0 defines your encoding of the chromosome&#39;s genes for your use-case. In the example above, we define a Binary Gene Set of length 15. Genex uses this function to generate an initial population of Chromosomes matching your encoding. Fitness Function def fitness_function(chromosome) do Enum.sum(chromosome.genes) end fitness_function/1 defines how the algorithm evaluates the fitness of a chromosome. It takes in a chromosome struct and returns a number. Fitness is how your algorithm determines which chromosomes should be persisted to the next generation as well as which chromosomes should be selected to crossover. In this case, we want to maximize 1&#39;s in our set of genes, so we define fitness as the sum of genes. Termination Criteria def terminate?(population) do population.max_fitness == 15 end terminate?/1 defines the termination criteria for your algorithm. This tells Genex when your algorithm should stop running. In this case we use a Max Fitness; however, you can also tell the algorithm to stop after a certain number of generations. Running Once you have defined an implementation module. Utilize the run/0 function to run the algorithm. The function will return the solution population for analysis. You can display a summary of the solution with the: Genex.Visualizers.Text.display_summary/1 function. soln = OneMax.run() Genex.Visualizers.Text.display_summary(soln) Configuration Genex offers a number of settings to adjust the algorithm to your liking. You can adjust: strategies, rates, and more. See the configuration guide for a full list. Customization You can customize every step of your genetic algorithm utilizing one or more of the many callbacks Genex provides. See the customization guide for more."},{"ref":"Genex.html#c:crossover/1","title":"Genex.crossover/1","type":"callback","doc":"Crossover a number of individuals to create a new population."},{"ref":"Genex.html#c:crossover_rate/1","title":"Genex.crossover_rate/1","type":"callback","doc":"Crossover rate as a function of the population."},{"ref":"Genex.html#c:encoding/0","title":"Genex.encoding/0","type":"callback","doc":"Generates a random gene set."},{"ref":"Genex.html#c:evaluate/1","title":"Genex.evaluate/1","type":"callback","doc":"Evaluates a population&#39;s fitness."},{"ref":"Genex.html#c:fitness_function/1","title":"Genex.fitness_function/1","type":"callback","doc":"Calculates a Chromosome&#39;s fitness."},{"ref":"Genex.html#c:mutate/1","title":"Genex.mutate/1","type":"callback","doc":"Mutate a number of individuals, depending on the mutation_rate."},{"ref":"Genex.html#c:mutation_rate/1","title":"Genex.mutation_rate/1","type":"callback","doc":"Mutation rate as a function of the population."},{"ref":"Genex.html#c:radiation/1","title":"Genex.radiation/1","type":"callback","doc":"Radiation level (aggressiveness) of mutation as a function of the population."},{"ref":"Genex.html#c:seed/0","title":"Genex.seed/0","type":"callback","doc":"Seeds a population."},{"ref":"Genex.html#c:select_parents/1","title":"Genex.select_parents/1","type":"callback","doc":"Selects a number of individuals for crossover, depending on the crossover_rate."},{"ref":"Genex.html#c:select_survivors/1","title":"Genex.select_survivors/1","type":"callback","doc":"Select a number of individuals to survive to the next generation, depending on the crossover_rate.s"},{"ref":"Genex.html#c:statistics/0","title":"Genex.statistics/0","type":"callback","doc":"Specifies the statistics to collect on the population."},{"ref":"Genex.html#c:terminate?/1","title":"Genex.terminate?/1","type":"callback","doc":"Tests the population for some termination criteria."},{"ref":"Genex.html#valid_rate?/1","title":"Genex.valid_rate?/1","type":"macro","doc":""},{"ref":"Genex.Support.Statistics.html","title":"Genex.Support.Statistics","type":"module","doc":"Provides basic summary statistics for use in the Population struct."},{"ref":"Genex.Support.Statistics.html#max/1","title":"Genex.Support.Statistics.max/1","type":"function","doc":"Takes the max of list. Returns Number. Parameters list: List of values. Examples iex&gt; list = [1, 2, 3, 4, 5] iex&gt; max(list) 5 iex&gt; list = [] iex&gt; max(list) nil"},{"ref":"Genex.Support.Statistics.html#mean/1","title":"Genex.Support.Statistics.mean/1","type":"function","doc":"Takes the mean of list. Returns Number. Parameters list: List of values. Examples iex&gt; list = [1, 2, 3, 4, 5] iex&gt; mean(list) 3.0 iex&gt; list = [] iex&gt; mean(list) 0"},{"ref":"Genex.Support.Statistics.html#min/1","title":"Genex.Support.Statistics.min/1","type":"function","doc":"Takes the min of list. Returns Number. Parameters list: List of values. Examples iex&gt; list = [1, 2, 3, 4, 5] iex&gt; min(list) 1 iex&gt; list = [] iex&gt; min(list) nil"},{"ref":"Genex.Support.Statistics.html#stdev/1","title":"Genex.Support.Statistics.stdev/1","type":"function","doc":"Takes the standard deviation of list. Returns Number. Parameters list: List of values. Examples iex&gt; list = [0, 1, 1, 1] iex&gt; stdev(list) 0.5 iex&gt; list = [] iex&gt; stdev(list) 0"},{"ref":"Genex.Support.Statistics.html#variance/1","title":"Genex.Support.Statistics.variance/1","type":"function","doc":"Takes the variance of list. Returns Number. Parameters list: List of values. Examples iex&gt; list = [0, 1, 1, 1] iex&gt; variance(list) 0.25 iex&gt; list = [] iex&gt; variance(list) 0"},{"ref":"Genex.Visualizers.Text.html","title":"Genex.Visualizers.Text","type":"module","doc":"Text visualizations of the Algorithm."},{"ref":"Genex.Visualizers.Text.html#display_summary/1","title":"Genex.Visualizers.Text.display_summary/1","type":"function","doc":"Display a summary of the population."},{"ref":"Genex.Visualizers.Text.html#init/0","title":"Genex.Visualizers.Text.init/0","type":"function","doc":"Display Initial Algorithm Text."},{"ref":"Genex.Operators.Crossover.html","title":"Genex.Operators.Crossover","type":"module","doc":"Implementation of several popular crossover methods. Crossover is analagous to reproduction or biological crossover. Genex utilizes pairs of chromosomes to create offspring from the genetic material of parent chromosomes. Crossover happens with some probability P(c). Typically this is a high probability. The probability of crossover or crossover_rate as it is called in our case, determines the number of parents selected to breed for the next generation. See more on this in the Selection documentation. Crossover operators are generic. As with any optimization problem, no single method will be perfect. Genex offers a variety of crossover operators to experiment with; however, you may find that you need to write your own to fit your specific use case. You can do this by overriding the crossover method. Each time a crossover takes place, 2 new children are created. These children then populate the children field of the Population struct before they are merged into the new population."},{"ref":"Genex.Operators.Crossover.html#blend/3","title":"Genex.Operators.Crossover.blend/3","type":"function","doc":"Performs a blend crossover. This will blend genes according to some alpha between 0 and 1. Returns {%Chromosome{}, %Chromosome{}}. Parameters p1: Parent one. p2: Parent two. alpha: Float between 0 and 1 representing percentage of each parent to blend into children."},{"ref":"Genex.Operators.Crossover.html#davis_order/2","title":"Genex.Operators.Crossover.davis_order/2","type":"function","doc":"Performs Davis Order crossover of a random slice. Returns {%Chromosome{}, %Chromosome{}}. Parameters p1: Parent one. p2: Parent two."},{"ref":"Genex.Operators.Crossover.html#messy_single_point/2","title":"Genex.Operators.Crossover.messy_single_point/2","type":"function","doc":"Performs a messy single point crossover at random points. This crossover disregards the length of the chromosome and will often arbitrarily increase or decrease it&#39;s size. Returns {%Chromosome{}, %Chromosome{}}. Parameters p1: Parent one. p2: Parent two."},{"ref":"Genex.Operators.Crossover.html#messy_single_point/4","title":"Genex.Operators.Crossover.messy_single_point/4","type":"function","doc":"Performs a messy single point crossover at known points. This crossover disregards the length of the chromosome and will often arbitrarily increase or decrease it&#39;s size. Returns {%Chromosome{}, %Chromosome{}}. Parameters p1: Parent one. p2: Parent two. point1: p1 split point. point2: p2 split point. Examples iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c2 = %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5} iex&gt; messy_single_point(c1, c2, 2, 4) {%Elixir.Genex.Chromosome{genes: [1, 2, 10], size: 3}, %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 3, 4, 5], size: 7}} iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c2 = %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5} iex&gt; messy_single_point(c1, c2, 0, 10) {%Elixir.Genex.Chromosome{genes: [], size: 0}, %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10, 1, 2, 3, 4, 5], size: 10}} iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c2 = %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5} iex&gt; messy_single_point(c1, c2, -2, 3) {%Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5}, %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5}}"},{"ref":"Genex.Operators.Crossover.html#simulated_binary/3","title":"Genex.Operators.Crossover.simulated_binary/3","type":"function","doc":"Performs a simulated binary crossover. Returns {%Chromosome{}, %Chromosome{}}. Parameters p1: Parent one. p2: Parent two. eta: Float"},{"ref":"Genex.Operators.Crossover.html#single_point/2","title":"Genex.Operators.Crossover.single_point/2","type":"function","doc":"Performs single point crossover at a random point. This will swap a random slice of genes from each chromosome, producing 2 new chromosomes. Returns {%Chromosome{}, %Chromosome{}}. Parameters p1: Parent one. p2: Parent two."},{"ref":"Genex.Operators.Crossover.html#single_point/3","title":"Genex.Operators.Crossover.single_point/3","type":"function","doc":"Performs single point crossover at a known point. This will swap a known slice of genes from each chromosome, producing 2 new chromosomes. Returns {%Chromosome{}, %Chromosome{}}. Parameters p1: Parent one. p2: Parent two. point: Slice to swap. Examples iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c2 = %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5} iex&gt; single_point(c1, c2, 2) {%Elixir.Genex.Chromosome{genes: [1, 2, 8, 9, 10], size: 5}, %Elixir.Genex.Chromosome{genes: [6, 7, 3, 4, 5], size: 5}} iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c2 = %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5} iex&gt; single_point(c1, c2, 10) {%Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5}, %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5}} iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c2 = %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5} iex&gt; single_point(c1, c2, 0) {%Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5}, %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5}} iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c2 = %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5} iex&gt; single_point(c1, c2, -5) {%Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5}, %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5}}"},{"ref":"Genex.Operators.Crossover.html#two_point/2","title":"Genex.Operators.Crossover.two_point/2","type":"function","doc":"Performs two-point crossover at a random point. This will swap two random slices of genes from each chromosome, producing 2 new chromosomes. Returns %Chromosome{}. Parameters p1: Parent one. p2: Parent two."},{"ref":"Genex.Operators.Crossover.html#two_point/4","title":"Genex.Operators.Crossover.two_point/4","type":"function","doc":"Performs two-point crossover at a known point. This will swap a known slice of genes from each chromosome, producing 2 new chromosomes. Returns {%Chromosome{}, %Chromosome{}}. Parameters p1: Parent one. p2: Parent two. first: First Split Point. second: Second Split Point. Examples iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c2 = %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5} iex&gt; two_point(c1, c2, 1, 3) {%Elixir.Genex.Chromosome{genes: [1, 7, 8, 4, 5], size: 5}, %Elixir.Genex.Chromosome{genes: [6, 2, 3, 9, 10], size: 5}} iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c2 = %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5} iex&gt; two_point(c1, c2, 10, 3) {%Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5}, %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5}} iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c2 = %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5} iex&gt; two_point(c1, c2, -3, 3) {%Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5}, %Elixir.Genex.Chromosome{genes: [6, 7, 8, 9, 10], size: 5}}"},{"ref":"Genex.Operators.Crossover.html#uniform/3","title":"Genex.Operators.Crossover.uniform/3","type":"function","doc":"Performs uniform crossover. This will swap random genes from each chromosome according to some specified rate, producing 2 new chrmosomes. Returns Chromosome. Parameters p1: Parent one. p2: Parent two. rate: Float between 0 and 1 representing rates to swap genes."},{"ref":"Genex.Operators.Mutation.html","title":"Genex.Operators.Mutation","type":"module","doc":"Implementation of several population mutation methods. Mutation takes place according to some rate. Mutation is useful for introducing novelty into the population. This ensures your solutions don&#39;t prematurely converge. Future versions of Genex will provide the ability to define the &quot;aggressiveness&quot; of mutations. As of this version of Genex, mutations effect the ENTIRE chromosome."},{"ref":"Genex.Operators.Mutation.html#bit_flip/2","title":"Genex.Operators.Mutation.bit_flip/2","type":"function","doc":"Perform a bit-flip mutation. This mutation performs a binary XOR on every gene in the Chromosome. Returns Chromosome. Parameters chromosome- Chromosome to mutate."},{"ref":"Genex.Operators.Mutation.html#gaussian/2","title":"Genex.Operators.Mutation.gaussian/2","type":"function","doc":"Performs a gaussian mutation. This mutation generates a random number at random genes in the chromosome. The random number is from a normal distribution produced from the mean and variance of the genes in the chromosome. Returns Chromosome. Parameters chromosome- Chromosome to mutate. radiation- Aggressiveness of mutation."},{"ref":"Genex.Operators.Mutation.html#invert/2","title":"Genex.Operators.Mutation.invert/2","type":"function","doc":"Perform inversion mutation on a random slice. This mutation reverses (inverts) a random slice of genes of the Chromosome. Returns %Chromosome{}. Parameters chromosome- Chromosome to mutate. radiation- Aggressiveness of mutation"},{"ref":"Genex.Operators.Mutation.html#invert/3","title":"Genex.Operators.Mutation.invert/3","type":"function","doc":"Perform inversion mutation on a known slice. This mutation reverses (inverts) a known slice of genes of the Chromosome. Returns %Chromosome{}. Parameters chromosome: Chromosome to mutate. start: Start of slice. finish: End of slice. Examples iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; invert(c1, 2, 4) %Elixir.Genex.Chromosome{genes: [1, 2, 4, 3, 5], size: 5} iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; invert(c1, 10, 3) %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; invert(c1, 0, 10) %Elixir.Genex.Chromosome{genes: [5, 4, 3, 2, 1], size: 5}"},{"ref":"Genex.Operators.Mutation.html#polynomial_bounded/5","title":"Genex.Operators.Mutation.polynomial_bounded/5","type":"function","doc":"Performs Polynomial Bounded mutation. See NSGA-II algorithm. Returns Chromosome. Parameters chromosome- Chromosome to mutate. radiation eta low high"},{"ref":"Genex.Operators.Mutation.html#scramble/2","title":"Genex.Operators.Mutation.scramble/2","type":"function","doc":"Perform a scramble mutation. This mutation shuffles the genes of the Chromosome between 2 random points. Returns %Chromosome{}. Parameters chromosome: Chromosome to mutate. radiation: Aggressiveness of the mutation."},{"ref":"Genex.Operators.Mutation.html#scramble/3","title":"Genex.Operators.Mutation.scramble/3","type":"function","doc":"Performs a scramble mutation of a known slice. This mutation shuffles the genes of a Chromosome between 2 known points. Parameters chromosome: Chromosome to mutate. start: First scramble point. finish: Second scramble point. Examples iex&gt; c1 = %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5} iex&gt; scramble(c1, 10, 4) %Elixir.Genex.Chromosome{genes: [1, 2, 3, 4, 5], size: 5}"},{"ref":"Genex.Operators.Mutation.html#uniform_integer/4","title":"Genex.Operators.Mutation.uniform_integer/4","type":"function","doc":"Performs uniform integer mutation. This mutation generates a random number at random genes in the chromosome. Returns Chromosome. Parameters chromosome- Chromosome to mutate. radiation- Aggressiveness of mutation. min- lower bound max- upper bound"},{"ref":"Genex.Operators.Selection.html","title":"Genex.Operators.Selection","type":"module","doc":"Implementation of several popular selection methods. Selection occurs in two stages in Genex: parent selection and survivor selection. Parent Selection dictates which chromosomes are to be reserved for crossover according to some crossover rate. In this stage, a number of chromosomes are selected and paired off in 2-tuples in the order they are selected. Future versions of Genex will provide more advanced methods of parent selection. Survivor Selection occurs last in the GA cycle. As of this version of Genex, the survivor rate is always equal to 1 - CR where CR is the crossover rate. Future versions will support more advanced survivor selection, including the ability to fluctuate the population according to some operators."},{"ref":"Genex.Operators.Selection.html#natural/2","title":"Genex.Operators.Selection.natural/2","type":"function","doc":"Natural selection of some number of chromosomes. This will select the n best (fittest) chromosomes. Returns Enum.t. Parameters chromosomes: Enum of Chromosomes. n: Number of chromosomes to select."},{"ref":"Genex.Operators.Selection.html#random/2","title":"Genex.Operators.Selection.random/2","type":"function","doc":"Random selection of some number of chromosomes. This will select n random chromosomes. Returns Enum.t. Parameters chromosomes: Enum of Chromosomes. n: Number of chromosomes to select."},{"ref":"Genex.Operators.Selection.html#roulette/2","title":"Genex.Operators.Selection.roulette/2","type":"function","doc":"Roulette selection of some number of chromosomes. This will select n chromosomes using a &quot;roulette&quot; wheel where the probability of a chromosome being selected is proportional to it&#39;s fitness. Returns Enum.t(). Parameters chromosomes: Enum of Chromosomes. n: Number of chromosomes to select."},{"ref":"Genex.Operators.Selection.html#stochastic_universal_sampling/2","title":"Genex.Operators.Selection.stochastic_universal_sampling/2","type":"function","doc":"Stochastic Universal Sampling of chromosomes. This will sample all of the chromosomes without bias, choosing them at evenly spaced intervals. Returns Enum.t(). Parameters chromosomes: Enum of Chromosomes. n: Number of chromomsomes to select."},{"ref":"Genex.Operators.Selection.html#tournament/3","title":"Genex.Operators.Selection.tournament/3","type":"function","doc":"Tournament selection of some number of chromosomes. This will select n chromosomes from tournaments of size k. We randomly select k chromosomes from the population and choose the max to be in the tournament. Returns Enum.t(). Parameters chromosomes: Enum of Chromosomes. n: Number of chromosomes to select. tournsize: The size of the tournament to run."},{"ref":"Genex.Operators.Selection.html#worst/2","title":"Genex.Operators.Selection.worst/2","type":"function","doc":"Worst selection of some number of chromosomes. This will select the n worst (least fit) chromosomes. Returns Enum.t. Parameters chromosomes: Enum of Chromosomes. n: Number of chromosomes to select."},{"ref":"Genex.Support.Genealogy.html","title":"Genex.Support.Genealogy","type":"module","doc":"Implementation of a genealogy tree. We use the Genealogy tree to model the history of the population from it&#39;s initialization to the end of the algorithm. The tree itself is an erlang digraph. An edge emanates from parent and is incident on child."},{"ref":"Genex.Support.Genealogy.html#add_generation/2","title":"Genex.Support.Genealogy.add_generation/2","type":"function","doc":"Add a generation of Chromosomes to Genealogy Tree. Returns Graph. Parameters genealogy - Reference to a Genealogy tree. chromosome - Chromosome to add."},{"ref":"Genex.Support.Genealogy.html#export/1","title":"Genex.Support.Genealogy.export/1","type":"function","doc":"Exports the genealogy tree."},{"ref":"Genex.Support.Genealogy.html#init/0","title":"Genex.Support.Genealogy.init/0","type":"function","doc":"Initializes a new Genealogy Tree. Returns Graph."},{"ref":"Genex.Support.Genealogy.html#update/2","title":"Genex.Support.Genealogy.update/2","type":"function","doc":"Updates a Genealogy Tree with just a vertex. Returns Graph. Parameters genealogy - Reference to a Genealogy Tree. chromosome - Chromosome to add to Genealogy."},{"ref":"Genex.Support.Genealogy.html#update/4","title":"Genex.Support.Genealogy.update/4","type":"function","doc":"Updates a Genealogy Tree with a vertex and it&#39;s parents. Returns Graph. Parameters genealogy - Reference to a Genealogy Tree. child - Chromosome to add. parent_a - Child&#39;s parent. parent_b - Child&#39;s parent."},{"ref":"Genex.Chromosome.html","title":"Genex.Chromosome","type":"module","doc":"A collection of genes. A Chromosome represents one solution to the problem you are trying to solve. Solutions are encoded into an enumerable of genes. The Chromosome is then evaluated based on some criteria you define."},{"ref":"Genex.Chromosome.html#get_fitness/1","title":"Genex.Chromosome.get_fitness/1","type":"function","doc":"Return the fitness of chromosome."},{"ref":"Genex.Chromosome.html#t:t/0","title":"Genex.Chromosome.t/0","type":"type","doc":""},{"ref":"Genex.Population.html","title":"Genex.Population","type":"module","doc":"A collection of Chromosomes. The Population represents the pool of solutions you are trying to optimize. Ideally, you persist the &#39;fittest&#39; features from generation to generation until you converge on a solution. The Population struct contains a number of useful fields that can be analyzed after the algorithm runs."},{"ref":"Genex.Population.html#sort/2","title":"Genex.Population.sort/2","type":"function","doc":"Sort the population by fitness."},{"ref":"Genex.Population.html#t:t/0","title":"Genex.Population.t/0","type":"type","doc":""},{"ref":"introduction-overview.html","title":"Overview","type":"extras","doc":"Overview Genex is a framework for writing Genetic Algorithms in Elixir. It provides methods and implementations mirroring each step in a typical Genetic Algorithm."},{"ref":"introduction-overview.html#genetic-algorithms","title":"Overview - Genetic Algorithms","type":"extras","doc":"A Genetic Algorithm is a search-based optimization technique based on the principles of Natural Selection and Evolution. Solutions are encoded as Chromosomes or Individuals with individual characteristics known as &quot;genes.&quot; A Genetic Algorithm typically starts with some initial population. This initial population represents a set of solutions or &quot;guesses&quot; to the problem. It then repeatedly applies a cycle of evolutionary operators on the population - producing newer generations of the population until some termination criteria is met. The basic outline is: Initialize Population Evaluate Population Apply Evolutionary Operators Repeat Until Termination Criteria Genex follows the same basic outline and exposes several helpful methods and configuration options to make writing Genetic Algorithms trivial."},{"ref":"introduction-overview.html#configuration","title":"Overview - Configuration","type":"extras","doc":"For information on how to configure Genex, please see the configuration guide."},{"ref":"introduction-overview.html#customization","title":"Overview - Customization","type":"extras","doc":"For information on how to customize Genex, please see the customization guide."},{"ref":"introduction-overview.html#tutorials","title":"Overview - Tutorials","type":"extras","doc":"Getting Started"},{"ref":"introduction-overview.html#guides","title":"Overview - Guides","type":"extras","doc":"To contribute a guide, please submit a pull request to the genex project on GitHub."},{"ref":"introduction-installation.html","title":"Installation","type":"extras","doc":"Installation To install Genex, add an entry to your mix.exs: def deps do [ # ... {:genex, &quot;~&gt; 0.1.0&quot;} ]"},{"ref":"introduction-installation.html#implementation-module","title":"Installation - Implementation Module","type":"extras","doc":"Genex requires an implementation module. Please see the Getting Started guide for more information."},{"ref":"introduction-installation.html#configuration","title":"Installation - Configuration","type":"extras","doc":"Genex offers a variety of configuration options. Please see the configuration guide for more information."},{"ref":"introduction-configuration.html","title":"Configuration","type":"extras","doc":"Configuration Genex offers a number of configuration options. This is a comprehensive list. To adjust radiation, crossover, and mutation rates, please see the Customization guide. Configuration options are provided after the use Genex line in your module. defmodule MyGeneticAlgorithm do use Genex, crossover_type: :two_point, mutation_type: :invert ... end"},{"ref":"introduction-configuration.html#defaults","title":"Configuration - Defaults","type":"extras","doc":":population_size: 100 :crossover_type: :single_point :parent_selection: :natural :survivor_selection: :natural :mutation_type: :scramble :minimize: false"},{"ref":"introduction-configuration.html#crossover_type","title":"Configuration - crossover_type","type":"extras","doc":"This option determines what kind of Crossover to use. For a detailed explanation of each Crossover method, see the Crossover guide. :single_point :two_point :uniform (requires :uniform_crossover_rate parameter set) :blend (requires :alpha parameter set) :simulated_binary (requires :eta parameter set) :messy_single_point :davis_order"},{"ref":"introduction-configuration.html#parent_selection-and-survivor_selection","title":"Configuration - parent_selection and survivor_selection","type":"extras","doc":"This option determines what kind of Selection to use. For a detailed explanation of each Selection method, see the Selection guide. :natural :random :worst :roulette :tournament (requires :tournsize parameter set) :stochastic"},{"ref":"introduction-configuration.html#mutation_type","title":"Configuration - mutation_type","type":"extras","doc":"This option determines what kind of mutation to use. For a detailed explanation of each Mutation method, see the Mutation guide. :bit_flip :invert :scramble :gaussian :uniform_integer (requires upper_bound and lower_bound parameters set) :polynomial_bounded (requires upper_bound, lower_bound and eta parameters set) :none"},{"ref":"introduction-configuration.html#minimize","title":"Configuration - minimize","type":"extras","doc":"This option tells Genex if you want to minimize or maximize fitness. When set to true, Genex will sort population fitness in ascending order. The &quot;strongest&quot; chromosome will be the one with the lowest fitness."},{"ref":"introduction-customization.html","title":"Customization","type":"extras","doc":"Customization Genex offers a number of operators and strategies for crafting Genetic Algorithms; however, there is often a need to have full control over the process. This guide is an attempt to illustrate the flexibility and power offered by Genex."},{"ref":"introduction-customization.html#configuration","title":"Customization - Configuration","type":"extras","doc":"Genex offers basic customization through a number of configuration options. Please see the Configuration guide for more information."},{"ref":"introduction-customization.html#rates","title":"Customization - Rates","type":"extras","doc":"In some cases, you will need to customize rates based on some population characteristics. Genex uses 3 rates throughout the GA life cycle: crossover_rate, mutation_rate, radiation. Crossover Rate The crossover rate dictates the number of parents to breed during the crossover phase. This number n equals Floor(Crossover Rate * Population Size). The default crossover rate is 0.75. This number means 75 percent of the chromosomes in the population are selected for crossover during each generation. The crossover rate can be changed by overriding the crossover_rate/1 function in your implementation module. The function accepts a population struct and returns a float between 0 and 1. defmodule MyGA do use Genex ... def crossover_rate(population), do: 0.60 end Mutation Rate The mutation rate dictates the number of chromosomes mutated during each generation. This number n equals Floor(Mutation Rate * Population Size). The default mutation rate is 0.05. This number means 5 percent of the chromosomes in the population are mutated during each generation. The mutation rate can be changed by overriding the mutation_rate/1 function in your implementation module. The function accepts a population struct and returns a float between 0 and 1. defmodule MyGA do use Genex ... def mutation_rate(population), do: 0.10 end Radiation The radiation level dictates the &quot;aggressiveness&quot; of mutation. It is used to determine the number of genes in the chromosome that are altered when a chromosome is selected for mutation. The default radiation level is 0.5. This number means 50 percent of the chromosome on average will be changed during mutation. The radiation level can be changed by overriding the radiation/1 function in your implementation module. The function accepts a population struct and returns a float between 0 and 1. defmodule MyGA do use Genex ... def radiation(population), do: 0.25 end"},{"ref":"introduction-customization.html#seeding-the-population","title":"Customization - Seeding the Population","type":"extras","doc":"Constructing an initial population is the first step in the Genex life cycle. By default, Genex constructs a list of n chromosomes where n is equal to the population_size parameter. These chromosomes are constructed using the encoding/0 function to generate a random set of genes for each. Sometimes it&#39;s better to start with a known, pre-constructed population. This is entirely possible thanks to Genex&#39;s flexibility. This ability is especially useful when you have a population stored in a file or database. You can start with a pre-constructed population by overriding the seed/0 function in your implementation module. The function MUST return {:ok, %Population{}}. Please not that you also must initialize the history field of the population struct using Genealogy.init() as well as the size field of the population. defmodule MyGA do use Genex ... def seed do history = Genex.Genealogy.init() chromosomes = read_from_somewhere() pop = %Population{chromosomes: chromosomes, history: history, size: length(chromosomes)} {:ok, pop} end end"},{"ref":"introduction-customization.html#evaluating-the-population","title":"Customization - Evaluating the Population","type":"extras","doc":"While most Genetic Algorithms evaluate chromosomes individually based on some predefined fitness function, you may find yourself needing to adjust the evaluation strategy of your algorithm. To do so, override the evaluation/1 function in your implementation module. For best results, ensure your function assigns: fitness to each of the chromosomes, a strongest chromosome, and the max fitness for the population. The function MUST return {:ok, %Population{...}}. Please also note that in order to avoid changing other fields in the population struct inadvertently, you must use Elixir&#39;s Map update syntax. defmodule MyGA do use Genex ... def evaluate(population) do chromosomes = population.chromosomes |&gt; Enum.map(fn c -&gt; some_custom_evaluation_technique(c) end) strongest = Enum.max_by(chromosomes, &amp;Chromosome.get_fitness/1) pop = %Population{population | chromosomes: chromosomes, strongest: strongest, max_fitness: strongest.fitness} {:ok, pop} end end"},{"ref":"introduction-customization.html#parent-selection","title":"Customization - Parent Selection","type":"extras","doc":"Genex comes with a number of prepackaged selection operators. However, there are some limitations to what the library provides. Genex offers the ability to implement a customized parent selection phase. As of this version of Genex, there is no ability to grow or shrink the population by default. Customizing this function offers a way around this shortcoming. In order to customize parent selection, override the select_parents/1 function in your implementation module. Please note that your function MUST assign a List of Tuple of Chromosomes to the parents field of the returned population struct. The tuples MUST be 2-tuples and your function MUST return {:ok, %Population{}}. Please also note that in order to avoid changing other fields in the population struct inadvertently, you must use Elixir&#39;s Map update syntax. defmodule MyGA do use Genex ... def select_parents(population) do chromosomes = population.chromosomes n = floor(crossover_rate(population) * length(chromosomes)) parents = chromosomes |&gt; some_selection_method(n) |&gt; Enum.chunk_every(2, 2, :discard) |&gt; Enum.map(fn f -&gt; List.to_tuple(f) end) pop = %Population{population | parents: parents} {:ok, pop} end end"},{"ref":"introduction-customization.html#crossover","title":"Customization - Crossover","type":"extras","doc":"Genex comes with a number of prepackaged crossover operators. However, there are some limitations to what the library provides. Genex offers the ability to customize the crossover phase. In order to customize crossover, override the crossover/1 function in your implementation module. You can access the parents selected for crossover in the parents field of the population struct. Please note your function MUST assign a List of chromosomes to the children field of the population struct. The function MUST return {:ok, %Population{}}. Additionally, you may want to update the history by making calls to Genealogy.update/4 after each new child is created. Please also note that in order to avoid changing other fields in the population struct inadvertently, you must use Elixir&#39;s Map update syntax. defmodule MyGA do use Genex ... def crossover(population) do children = population.parents |&gt; Enum.map( fn {p1, p2} -&gt; {c1, c2} = some_crossover(p1, p2) end) Genealogy.update(population.history, c1, p1, p2) Genealogy.update(population.history, c2, p1, p2) [c1, c2] end |&gt; List.flatten() pop = %Population{population | children: children} {:ok, pop} end end"},{"ref":"introduction-customization.html#mutation","title":"Customization - Mutation","type":"extras","doc":"Genex comes with a number of mutation oeprators. However, there are some limitations to what the library provides. Genex offers the ability to customize the mutation phase. In order to customize mutation, override the mutate/1 function in your implementation module. You should access and modify the chromosomes field of the population struct. The function accepts a population struct. Your function MUST return {:ok, %Population{}}. *Please also note that in order to avoid changing other fields in the population struct inadvertently, you must use Elixir&#39;s Map update syntax. defmodule MyGA do use Genex ... def mutate(population) do chromosomes = population.chromosomes |&gt; Enum.map(some_mutation_function(mutation_rate())) pop = %Population{population | chromosomes: chromosomes} {:ok, pop} end end"},{"ref":"introduction-customization.html#survivor-selection","title":"Customization - Survivor Selection","type":"extras","doc":"Genex supports customization of the &quot;reinsertion&quot; phase of Genetic algorithms using it&#39;s select_survivors/1 method. Later versions of Genex will support defining &quot;kill criteria&quot; and other constraints; however, this method will allow you to customize how populations grow and decay over time. In order to customize survivor selection, override the select_survivors/1 method in your implementation module. This function accepts a population struct. To ensure smooth running of the rest of the algorithm, this function should populate the survivors field of the population struct it returns. This function MUST return {:ok, %Population{}}. defmodule MyGA do use Genex ... def select_survivors(population) do survivors = population.chromosomes |&gt; Enum.filter(fn c -&gt; c.age &lt; 5 end) pop = %Population{population | survivors: survivors} {:ok, pop} end end"},{"ref":"support-statistics.html","title":"Statistics","type":"extras","doc":"Statistics Genex offers the ability to track statistics about a population. It comes with a basic, default statistics package; however, you have the ability to extend it with your own 3rd party functions."},{"ref":"support-statistics.html#defaults","title":"Statistics - Defaults","type":"extras","doc":"By default Genex will collect the following information on a population during each generation: mean via Statistics.mean/1 variance via Statistics.variance/1 max via Statistics.max/1 min via Statistics.min/1 stdev via Statistics.stdev/1"},{"ref":"support-statistics.html#customization","title":"Statistics - Customization","type":"extras","doc":"You can customize the statistics collected on the population with your own methods by overriding the statistics/0 function in your implementation module. Your function must return a Keyword list where the key represents the statistic and the value is a reference to the function you want to run. As of this version of Genex, your functions MUST accept a List of numbers. def MyGA do use Genex ... def statistics do [ mean: &amp;MyPackage.my_mean/1, stdev: &amp;Statistics.stdev/1 ] end end"},{"ref":"support-statistics.html#logging-and-exporting","title":"Statistics - Logging and Exporting","type":"extras","doc":"The ability to log and export statistics across generations is coming in future versions of Genex. Stay tuned!"},{"ref":"support-benchmarking.html","title":"Benchmarking","type":"extras","doc":"Benchmarking Genex offers the ability to benchmark your application through the use of benchmark/0. This function will use Benchee to benchmark the runtime of each of the functions in your Genetic algorithm. The current benchmark function simply runs each function using Benchee.run/0 with default settings. The output will display in your console."},{"ref":"support-benchmarking.html#customization","title":"Benchmarking - Customization","type":"extras","doc":"There are many reasons you&#39;d want to customize the benchmarking features of Genex. Perhaps you prefer another benchmarking tool or you want to only benchmark one function. Regardless, Genex makes this all possible. You can customize the benchmarking features by overriding the benchmark/0 function in your implementation module. You can then utilize your own benchmarking module or package, or change the settings of passed to Benchee. defmodule MyGA do use Genex ... def benchmark do Benchee.run(%{ &quot;seed/0&quot; =&gt; fn -&gt; seed() end }) end end"},{"ref":"support-genealogy.html","title":"Genealogy","type":"extras","doc":"Genealogy Genex tracks the complex Genealogy of your algorithm&#39;s populations using libgraph. The Genealogy tree is implemented as a directed graph with edges emanating from parents and incident on children. The Genealogy tree is available in the history field of a Population struct. Take a look at libgraph&#39;s API for a number of useful functions for operating on graph structure."},{"ref":"support-genealogy.html#exporting-and-visualization","title":"Genealogy - Exporting and Visualization","type":"extras","doc":"Genex offers the ability to export the Genealogy tree for analysis later on. So far, exports can only be done to DOT files. Here&#39;s how: Assuming you have an implementation module defined, run your algorithm: soln = MyGA.run() Now, call Genex.Support.Genealogy.export/1 to export your genealogy tree. {:ok, binary} = Genex.Support.Genealogy.export(soln.history) Pay close attention to the export/1 signature. It returns {:ok, binary} where binary is the raw text of your generated file. You can then use Elixir&#39;s built-in File IO to create your DOT file. You can use software like Graphviz to view your Genealogy tree. Please note: If your algorithm went on for thousands of generations, this export is going to take some time. Be patient!"},{"ref":"tutorials-getting-started.html","title":"Getting Started","type":"extras","doc":"Getting Started"},{"ref":"tutorials-getting-started.html#install-genex","title":"Getting Started - Install Genex","type":"extras","doc":"Genex requires Elixir v1.9. To install Genex, simply add it to your mix.exs: def deps do [ # ... {:genex, &quot;~&gt; 0.1.0&quot;} ] end"},{"ref":"tutorials-getting-started.html#create-implementation-module","title":"Getting Started - Create Implementation Module","type":"extras","doc":"Genex requires an implementation module. The implementation module requires 3 functions: encoding/0, fitness_function/1, and terminate?/1. Consider the implementation module below: defmodule OneMax do use Genex def encoding, do: for _ &lt;- 1..15, do: Enum.random(0..1) def fitness_function(c), do: Enum.sum(c.genes) def terminate?(p), do: p.max_fitness &gt;= 15 end The module we just defined is an implementation of the &quot;One Max&quot; problem. The One Max Problem is a very basic application of Genetic Algorithms in which we attempt to maximize the number of 1&#39;s in a list of 1&#39;s and 0&#39;s. The problem itself is trivial; however, it provides a good introduction to Genetic Algorithms. Let&#39;s take a closer look at the module, line-by-line. use Genex This is how we tell Elixir we want to implement a Genetic Algorithm using Genex. We didn&#39;t include any configuration options. This tells Genex we want to use the defaults. Please see the configuration guide for a list of options and defaults. Encoding def encoding, do: for _ &lt;- 1..15, do: Enum.random(0..1) Before the algorithm starts, Genex will generate a &quot;seed&quot; population for us. To do so, it needs an encoding of a solution to your problem. This encoding can be ANYTHING as long as it implements the Enumerable protocol. For the purposes of our algorithm, we encode our solutions as a List of 1&#39;s and 0&#39;s. The list has length 15. Fitness Function def fitness_function(c), do: Enum.sum(c.genes) We now need to tell Genex how to evaluate our solutions. This is where the fitness function comes into play. A fitness function takes a Chromosome and assigns some value to it. Conventionally, higher fitness means a &quot;stronger&quot; solution. Usually, we want to maximize fitness. In our case, the fitness is just the sum of the genes. This effectively counts the number of 1&#39;s in the list. Termination Criteria def terminate?(p), do: p.max_fitness &gt;= 15 Finally, Genex needs to know when to stop the algorithm. We define a termination criteria based on some characteristic in the population. For our purposes, we&#39;ve told Genex to terminate the program when the population has reached a max_fitness of 15. Some algorithms need to terminate after a number of generations. This is possible as well. The Population struct exposes a bunch of useful information about the algorithm."},{"ref":"tutorials-getting-started.html#running-our-algorithm","title":"Getting Started - Running Our Algorithm","type":"extras","doc":"Now that we&#39;ve defined our implementation module, there are a number of ways to run the algorithm. In a Script This is often the easiest solution. With your module defined in a mix project, create a .exs file and insert the following line: OneMax.run() Then, in the console: mix run path/to/script.exs You should see the algorithm running with a summary of the current population. It&#39;s best to run the algorithms full screen, as right now the visualizers can be problematic. Using IEX Again with your module defined in a mix project, type the following into the console: iex -S mix When IEX loads, run the following: iex&gt; OneMax.run() You&#39;ll notice that a struct is returned. That&#39;s because run/0 returns the solution population. The solution population contains the final generation population that was produced before your termination criteria was met."},{"ref":"tutorials-getting-started.html#visualizers","title":"Getting Started - Visualizers","type":"extras","doc":"Genex contains a text-visualization model to view summaries of Populations. To view a summary of the population returned from the run/0 function, use: soln = OneMax.run() Genex.Visualizers.Text.display_summary(soln) This summary will show an overview of the population. Additionally, IO.inspect can come in handy when analyzing your algorithm."},{"ref":"tutorials-getting-started.html#issues","title":"Getting Started - Issues","type":"extras","doc":"Please report any issues on the http://www.github.com/seanmor5/genex Github page."},{"ref":"operators-crossover.html","title":"Crossover","type":"extras","doc":"Crossover The purpose of this guide is to give an overview of the different type of crossover methods available in the Genex library. Crossover or Recombination is the process of creating new solutions from selected old solutions. The idea is to eliminate weak solutions in favor of stronger ones in order to converge on an optimal value. Genex&#39;s crossover algorithms accept List of Tuple where each element in the Tuple is a parent. Genex stores the resulting children (2 per pair of parents) in the children field of the Population struct. All algorithms tested on 2 1,000,000-gene binary Chromosomes with Benchee."},{"ref":"operators-crossover.html#single-point","title":"Crossover - Single Point","type":"extras","doc":"Additional Options: Run Time: 85.10 ms Memory Usage: 31.36 MB Single Point Crossover is the simplest crossover algorithm and the default algorithm used by Genex. Genex uses Erlang&#39;s :rand package to generate a uniform random number between 0 and n-1, where n is the size of the chromosome. This number is the crossover point. Genex then splits each parent Chromosome at the crossover point and swaps the slices to create 2 new Chromosomes. The pseudocode below illustrates this process with an assumed crossover point of 3. parent a := [1, 2, 3, 4, 5] parent b := [6, 7, 8, 9, 10] crossover point := uniform random number between 0 and 4. child a := parent b[..3] + parent a[3..] child b := parent a[..3] + parent b[3..] Results: child a := [1, 2, 3, 9, 10] child b := [6, 7, 8, 4, 5] For most practical applications of Genetic Algorithms, Single Point Crossover is effectively useless. You should only use this for testing and learning purposes."},{"ref":"operators-crossover.html#two-point","title":"Crossover - Two Point","type":"extras","doc":"Additional Options: Run Time: 111.76 ms Memory Usage: 52.89 MB Two Point Crossover is very similar to Single Point Crossover. The name implies exactly what it does - executes a crossover with two points. Genex uses Erlang&#39;s :rand package to generate 2 random numbers between 0 and n-1 where n is the size of the chromosome. These 2 numbers are the crossover points. Genex then splits each parent chromosome into 3 slices: a head, a middle, and a tail. The middle slices are then swapped between the 2 chromosomes, creating 2 child chromosomes. The difference between Two Point and Single Point Crossover is that Two Point Crossover can crossover slices in the MIDDLE of the chromosome. The pseudocode below illustrates this process with assumed crossover points of 2 and 4. parent a := [1, 2, 3, 4, 5] parent b := [6, 7, 8, 9, 10] point 1 := uniform random number between 0 and 4 point 2 := uniform random number between 0 and 4 child a := parent a[..2] + parent b[2..4] + parent a[4..] child b := parent b[..2] + parent a[2..4] + parent b[4..] Results: child a := [1, 2, 8, 9, 5] child b := [6, 7, 3, 4, 10] Two Point Crossover, while slightly more useful than Single Point Crossover, likely will not be very effective in practical applications."},{"ref":"operators-crossover.html#uniform-crossover","title":"Crossover - Uniform Crossover","type":"extras","doc":"Additional Options: :uniform_crossover_rate Run Time: 914.47 ms Memory Usage: 247.95 MB Uniform Crossover is slightly more complex than our two previous crossover algorithms. It offers more flexibility and control over the crossover process and allows for the creation of more diverse chromosomes than Single and Two Point Crossover. In Uniform Crossover, the parent genes are zipped into pairs. We iterate through these pairs 1-by-1. Starting from the first pair, Genex generates random uniform numbers between 0 and 1 using Erlang&#39;s :rand package. If the number is less than the :uniform_crossover_rate, the genes are swapped. If not, the genes stay in place. Uniform Crossover creates 2 new Chromosomes with genes swapped in random places from the parent Chromosomes. A higher :uniform_crossover_rate will lead to more genes being swapped. The opposite is true for a lower rate. The pseudocode below illustrates this process with a :uniform_crossover_rate of 0.5. parent a := [1, 2, 3, 4, 5] parent b := [6, 7, 8, 9, 10] gene pairs := zip(parent a, parent b) for {x, y} in gene pairs: if(uniform random number &lt; 0.5): swap(x, y) {child a, child b} := unzip(gene pairs) Results: child a := [1, 7, 3, 9, 5] child b := [6, 2, 8, 4, 10] Uniform Crossover offers much more flexibility than Single and Two Point Crossover. This is likely the preferred method when dealing with Binary Chromosomes."},{"ref":"operators-crossover.html#blend","title":"Crossover - Blend","type":"extras","doc":"Additional Options: :alpha Run Time: 1578.04 ms Memory Usage: 259.40 MB Blend crossover blends the genes of two parent chromosomes according to the formula: C1 := ((1 - γ) * x) + (γ * y) C2 := (γ * x) + ((1 - γ) * y) γ := ((1 + (2 * α)) * μ) - α α := between 0 and 1 μ := uniform random number between 0 and 1 x := Parent A gene y := Parent B gene Blend Crossover uses alpha as well as some random numbers to dictate the amount of each of the respective parent&#39;s genes to blend into the offspring. Blend crossover is considerably slower than any of the aforementioned crossover strategies; however, it can be very effective with certain genotypes."},{"ref":"operators-crossover.html#davis-order","title":"Crossover - Davis Order","type":"extras","doc":"Additional Options: Run Time: 252.50 ms Memory Usage: 18.09 MB Davis Order Crossover or Order 1 Crossover deals with permutations. That is, when solutions are encoded as permutation where order determines the fitness of the chromosome, Davis Order Crossover is extremely useful. The algorithm is as follows: 1) Pick a random slice of genes from Parent A 2) Drop the slice in the same spot in Child 1 and mark out corresponding genes in Parent B. 3) Starting from the RIGHT after the corresponding genes in Parent B, insert remaining genes from Parent B into Child 1, wrapping around the chromosome if need be. 4) Repeat for Child 2 starting with Parent B. Davis Order is an exceptionally fast crossover method and is useful when solutions are encoded as permutations."},{"ref":"operators-crossover.html#messy-single-point-crossover","title":"Crossover - Messy Single Point Crossover","type":"extras","doc":"Additional Options: Run Time: 95.24 ms Memory Usage: 18.09 MB Messy Single Point Crossover is essentially the same algorithm as Single Point Crossover; however, it disregards the size of the chromosome and selects TWO crossover points - one for each parent. Messy Single Point Crossover will arbitrarily grow or shrink chromosomes. Crossover Points are selected at random for each parent. The chromosomes are then split at their respective crossover points and recombined to make two offspring. The pseudocode below shows this process: parent a := [1, 2, 3, 4, 5] parent b := [6, 7, 8, 9, 10] point 1 := uniform random number between 0 and 4 point 2 := uniform random number between 0 and 4 child a := parent b[..point2] + parent a[point1..] child b := parent a[..point1] + parent b[..point2] Results: child a := [6, 7, 8, 2, 3, 4, 5] child b := [1, 9, 10] Messy Single Point Crossover is extremely simple and can be useful in cases when you don&#39;t care about the length of your chromosomes. Beware that it is easy to end up with chromosomes of 0 length in this algorithm."},{"ref":"operators-crossover.html#simulated-binary","title":"Crossover - Simulated Binary","type":"extras","doc":"Additional Options: :eta Run Time: 1601.81 ms Memory Usage: 389.11 MB Simulated Binary Crossover combines the genes of two parents according to the following formula: C1 := 0.5 * ((1 + β) * x + (1 - β) * y) C2 := 0.5 * ((1 - β) * x + (1 + β) * y) β := (2 * p)^(1 / (η + 1)) OR β := (1 / (2 * (1 - p)))^(1 / (η + 1)) p := uniform random number η := between 0 and 1 It is used in the NSGA-II algorithm."}]